

/// runs on pretty much anything, I used a spare 3.3V / 8MHz arduino pro mini clone from my stash

#include <Adafruit_NeoPixel.h>

#define LED_PIN    9

// How many NeoPixels are attached to the Arduino?
#define LED_COUNT 1

Adafruit_NeoPixel pix(LED_COUNT, LED_PIN, NEO_RGB + NEO_KHZ800);

int masterSwitch = 2;
int slaveSwitch = 3;
int blueLed = 10;
int redLed = 11;

int black = pix.Color(0, 0, 0);
uint32_t red = pix.Color(255, 0, 0);
uint32_t blue = pix.Color(0, 0, 255);
uint32_t purple = pix.Color(255, 0, 255);
uint32_t green = pix.Color(0, 255, 0);
uint32_t pixColor;
uint32_t colorMap[4] = {black, blue, red, purple};
uint32_t colorCombo;

#define MONITORING_PERIOD       50
#define STABLE_PERIOD           500
bool blueLedState, redLedState, masterSwitchState;
uint32_t monitoringCounter, blueLedActivity, redLedActivity;
bool updatedBlue = false;
bool updatedRed = false;


void setup() {
  // put your setup code here, to run once:
  pinMode(masterSwitch, INPUT_PULLUP);
  pinMode(slaveSwitch, OUTPUT);
  pinMode(blueLed, INPUT);
  pinMode(redLed, INPUT);

  Serial.begin(115200);
  Serial.println("BLE module automation - Plecter Labs 2022");
  delay(100);

  pix.begin();           
  pix.show();            
  pix.setBrightness(120); // Set BRIGHTNESS to about 1/5 (max = 255)
  pix.setPixelColor(0, green); // Set pixel 'c' to value 'color'
  pix.show();
  digitalWrite(slaveSwitch, HIGH);
  int timeout = 6000;
  int ledState = digitalRead(blueLed);
  Serial.println("Waiting for module to turn on - Blue Led");
  while(ledState && timeout--) {
    delay(1);
    ledState = digitalRead(blueLed);
  }
  delay(100);
  digitalWrite(slaveSwitch, LOW);   // or turn as input
  Serial.println("BLE module started");
  monitoringCounter = millis();
}

void loop() {

  // state machine with monitoring master switch, and leds => reflected on neopixel

  if(millis() - monitoringCounter > MONITORING_PERIOD) {
    monitoringCounter = millis();
    if(blueLedState != !digitalRead(blueLed)) {
      blueLedState = !digitalRead(blueLed);   // stores state
      blueLedActivity = millis();
      updatedBlue = false;
      Serial.println("Blue led blinking - waiting for pairing / connection");
    }
    else {  // stable ? 
      if(millis() - blueLedActivity > STABLE_PERIOD && !updatedBlue) {
        Serial.println("BLUE LED STABLE");
        updatedBlue = true;
        if(!blueLedState) {
          Serial.println("BLUE OFF - Module most likely off");
        }
        else {
          Serial.println("BLUE ON - Module most likely ON and paired / connected");
        }
      }
    }
    if(redLedState != !digitalRead(redLed)) {
      redLedState = !digitalRead(redLed);   // stores state
      redLedActivity = millis();
      updatedRed = false;
      Serial.println("Red led blinking");
    }
    else {  // stable ? 
      if(millis() - redLedActivity > STABLE_PERIOD && !updatedRed) {
        Serial.println("BLUE LED STABLE");
        updatedRed = true;
        if(!redLedState) {
          Serial.println("RED OFF - Module most likely off");
        }
        else {
          Serial.println("RED ON - Module most likely Going Off or Reset pairing list");
        }
      }
    }
    
    redLedState = !digitalRead(redLed);
    colorCombo = redLedState;
    colorCombo <<= 1;
    colorCombo |= blueLedState;
    pixColor = colorMap[colorCombo];
    //Serial.println(colorCombo);
    pix.setPixelColor(0, pixColor); // Set pixel 'c' to value 'color'
    pix.show();

    masterSwitchState = !digitalRead(masterSwitch);
    digitalWrite(slaveSwitch, masterSwitchState);    
  }

}
